import logging
import logging.handlers
import os

from flask import Flask, render_template
from flask_sqlalchemy import SQLAlchemy

from . import config

# the version as used in setup.py and docs
__version__ = "1.2.0"

# Flask App
app = Flask(__name__)
# Is this an OpenShift deployment?
openshift = os.getenv('OPENSHIFT_PROD')

# set up basic logging so that initial messages are not lost (until we
# configure logging properly)
_logging_fmt = '[%(name)-15s] %(asctime)s %(levelname)-7s %(message)s'
_logging_fmt_debug = '[%(name)-23s:%(lineno)-3d] %(asctime)s %(levelname)-7s %(message)s'
_logging_datefmt = '%Y-%m-%d %H:%M:%S'
logging.basicConfig(format=_logging_fmt_debug, datefmt=_logging_datefmt)
logging.getLogger().setLevel(logging.DEBUG)

# load default configuration
if os.getenv('DEV') == 'true':
    app.logger.debug('Using development config')
    app.config.from_object('blockerbugs.config.DevelopmentConfig')
elif os.getenv('TEST') == 'true' or openshift == "0":
    app.logger.debug('Using testing config')
    app.config.from_object('blockerbugs.config.TestingConfig')
else:
    app.logger.debug('Using production config')
    app.config.from_object('blockerbugs.config.ProductionConfig')
if openshift:
    app.logger.debug('Using openshift config')
    config.openshift_config(app.config, openshift)

# load real configuration values to override defaults
config_file = '/etc/blockerbugs/settings.py'
if not app.testing:
    if os.path.exists(config_file):
        app.logger.info('Loading configuration from %s' % config_file)
        app.config.from_pyfile(config_file)
    else:
        config_file = os.path.abspath('conf/settings.py')
        if os.path.exists(config_file):
            app.logger.info('Loading configuration from %s' % config_file)
            app.config.from_pyfile(config_file)
        else:
            app.logger.info('No extra config found, using defaults')

# Being able to set DEBUG from env might be useful
if os.getenv('DEBUG') == 'true':
    app.config["DEBUG"] = True

# Make sure config URLs end with a slash, so that we have them all in an
# expected format
for key in ['BUGZILLA_URL', 'KOJI_URL', 'BODHI_URL', 'FAS_BASE_URL',
            'PAGURE_URL', 'PAGURE_API']:
    if not app.config[key].endswith('/'):
        app.config[key] = app.config[key] + '/'


def setup_logging():
    formatter = logging.Formatter(
        fmt=_logging_fmt_debug if app.debug else _logging_fmt,
        datefmt=_logging_datefmt)
    loglevel = logging.DEBUG if app.debug else logging.INFO

    root_logger = logging.getLogger()
    root_logger.setLevel(logging.DEBUG)
    root_logger.handlers.clear()
    app.logger.handlers.clear()

    if app.config['STREAM_LOGGING']:
        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(loglevel)
        stream_handler.setFormatter(formatter)
        root_logger.addHandler(stream_handler)
        app.logger.debug("doing stream logging")

    if app.config['SYSLOG_LOGGING']:
        syslog_handler = logging.handlers.SysLogHandler(
            address='/dev/log', facility=logging.handlers.SysLogHandler.LOG_LOCAL4)
        syslog_handler.setLevel(loglevel)
        syslog_handler.setFormatter(formatter)
        root_logger.addHandler(syslog_handler)
        app.logger.debug("doing syslog logging")

    if app.config['FILE_LOGGING'] and app.config['LOGFILE']:
        file_handler = logging.handlers.RotatingFileHandler(
            app.config['LOGFILE'], maxBytes=500000, backupCount=5)
        file_handler.setLevel(loglevel)
        file_handler.setFormatter(formatter)
        root_logger.addHandler(file_handler)
        app.logger.debug("doing file logging to %s" % app.config['LOGFILE'])


setup_logging()

# database
if app.config['SHOW_DB_URI']:
    app.logger.debug('using DBURI: %s' % app.config['SQLALCHEMY_DATABASE_URI'])
db = SQLAlchemy(app)

# "Hotfix" for proxy handling on current deployment, my guess is that the proxy
# server is set differently than it was, but what do I know...
if app.config["BEHIND_PROXY"]:
    from werkzeug.contrib.fixers import ProxyFix
    app.wsgi_app = ProxyFix(app.wsgi_app, num_proxies=1)


# === Flask views and stuff ===
@app.template_filter('tagify')
def tagify(value):
    return value.replace(' ', '')


@app.template_filter('urlsplit')
def urlsplit(value):
    return value.split('/')


@app.template_filter('getname')
def getname(value, entry=1):
    return value.split('/')[entry]


@app.template_filter('getsubname')
def getsubname(value):
    urlparts = value.split('/')
    if len(urlparts) >= 5:
        return ''.join(urlparts[2:4])
    else:
        return ''


@app.template_filter('updatetype')
def updatetype(update):
    is_fe = False
    for bug in update.bugs:
        if bug.proposed_blocker or bug.accepted_blocker:
            return 'blocker'
        elif bug.proposed_fe or bug.accepted_fe:
            is_fe = True
    if is_fe:
        return 'FE'


@app.template_filter('updatelabel')
def updatelabel(bug):
    from .models.update import Update
    label = []
    lowest_status_update = bug.updates.filter(
        Update.status != 'obsolete',
        Update.status != 'deleted',
        Update.status != 'unpushed'
    ).first()
    if lowest_status_update:
        if lowest_status_update.status == 'stable':
            if bug.status in ['MODIFIED', 'ON_QA', 'VERIFIED', 'CLOSED']:
                label.append('<span class="badge badge-success">')
            else:
                label.append('<span class="badge badge-info">')
            if lowest_status_update.pending:
                label.append('pending ')
            label.append('stable</span>')

        elif lowest_status_update.status == 'testing':
            label.append('<span class="badge badge-warning">')
            if lowest_status_update.pending:
                label.append('pending ')
            label.append('testing</span>')

    return ''.join(label)

@app.template_filter('datetime')
def datetime_format(value, format='%Y-%m-%d %H:%M:%S UTC'):
    if value is not None:
        return value.strftime(format)
    return ''


# register blueprints
from blockerbugs.controllers.main import main
app.register_blueprint(main)

from blockerbugs.controllers.admin import admin

from blockerbugs.controllers.api import api_v0
app.register_blueprint(api_v0)

# setup error handling
@app.errorhandler(404)
def page_not_found(e):
    return render_template('404.html', title="Page Not Found!"), 404
