import datetime

from blockerbugs.models.milestone import Milestone
from blockerbugs.models.release import Release
from blockerbugs.models.bug import Bug
from blockerbugs.models.update import Update
from blockerbugs.models.criterion import Criterion
from blockerbugs import db


def generate_release(number=99):
    return Release(number)

def generate_milestone(version, name, release, blocker_tracker=123456, accepted_tracker=234567):
    return Milestone(release, version, blocker_tracker, accepted_tracker, name)


def generate_bug(bugid, summary, milestone):
    return Bug(bugid, 'https://bugzilla.redhat.com/show_bug.cgi?id=%d' % bugid,
               summary, 'NEW', 'testcomponent', milestone, True, True, 'John Doe')

def generate_update(title, status, bugs, release, milestones, url='http://localhost/update'):
    return Update(title, url, 1, status, bugs,
                  release, milestones, date_submitted=datetime.datetime.utcnow() )


class TestfuncBugModel(object):
    def setup_method(self, method):
        db.session.rollback()
        db.drop_all()
        db.create_all()
        self.ref_release = generate_release()
        self.ref_milestone = generate_milestone('alpha', '99-alpha', self.ref_release)

    def teardown_method(self, method):
        db.session.rollback()
        db.session.close()
        db.drop_all()

    def test_add_release(self):
        ref_number = 99

        test_release = generate_release(ref_number)
        db.session.add(test_release)
        db.session.commit()

    def test_add_milestone(self):
        ref_version = 'alpha'
        ref_name = '99-alpha'

        # sqlalchemy tries to add the ref_milestone if we use ref_release here
        # so create a new one that has nothing to do with the setup objects
        test_release = generate_release()
        test_milestone = generate_milestone(ref_version, ref_name, test_release)
        db.session.add(test_release)
        db.session.add(test_milestone)
        db.session.commit()

        milestones = Milestone.query.filter_by(name=ref_name).all()
        assert len(milestones) == 1
        milestone = milestones[0]
        assert milestone.version == ref_version
        assert milestone.name == ref_name

    def test_add_bug(self):
        db.session.add(self.ref_release)
        db.session.add(self.ref_milestone)
        db.session.commit()

        ref_bugid = 123456
        ref_summary = 'testbug 1'
        testbug = generate_bug(ref_bugid, ref_summary, self.ref_milestone)

        db.session.add(testbug)
        db.session.commit()

        bugs = Bug.query.filter_by(bugid=ref_bugid).all()
        assert len(bugs) == 1
        bug = bugs[0]
        assert bug.bugid == ref_bugid
        assert bug.summary == ref_summary
        assert bug.milestone == self.ref_milestone

    def test_add_update(self):
        ref_bugid = 123456
        ref_bug = generate_bug(ref_bugid, 'testbug1', self.ref_milestone)
        db.session.add(self.ref_release)
        db.session.add(self.ref_milestone)
        db.session.add(ref_bug)
        db.session.commit()

        ref_title = 'test-1.0-1.fc99'
        ref_status = 'PENDING STABLE'
        ref_update = generate_update(ref_title, ref_status, [ref_bug],
                                     self.ref_release, [self.ref_milestone])

        db.session.add(ref_update)
        db.session.commit()

        updates = Update.query.filter_by(title=ref_title).all()
        assert len(updates) == 1
        update = updates[0]
        assert update.title == ref_title
        assert update.status == ref_status
        assert update.bugs[0] == ref_bug


    def test_add_update_long_title(self):
        ref_bugid = 123456
        ref_bug = generate_bug(ref_bugid, 'testbug1', self.ref_milestone)
        db.session.add(self.ref_release)
        db.session.add(self.ref_milestone)
        db.session.add(ref_bug)
        db.session.commit()

        ref_title = 'x' * 3000
        ref_url = 'http://localhost/update/%s' % ref_title
        ref_status = 'PENDING STABLE'
        ref_update = generate_update(ref_title, ref_status, [ref_bug],
                                     self.ref_release, [self.ref_milestone],
                                     url=ref_url)
        db.session.add(ref_update)
        db.session.commit()

        updates = Update.query.filter_by(title=ref_title).all()
        assert len(updates) == 1
        update = updates[0]
        assert update.title == ref_title
        assert update.url == ref_url


    def test_add_update_multiple_bugs(self):
        ref_bugs = [generate_bug(123456, 'testbug1', self.ref_milestone),
                    generate_bug(234567, 'testbug2', self.ref_milestone)]
        db.session.add(self.ref_release)
        db.session.add(self.ref_milestone)
        db.session.add(ref_bugs[0])
        db.session.add(ref_bugs[1])
        db.session.commit()

        ref_title = 'test-1.0-1.fc99'
        ref_status = 'PENDING STABLE'
        ref_update = generate_update(ref_title, ref_status, ref_bugs,
                                     self.ref_release, [self.ref_milestone])
        db.session.add(ref_update)
        db.session.commit()

        updates = Update.query.filter_by(title=ref_title).all()
        assert len(updates) == 1
        update = updates[0]
        assert len(update.bugs) == 2
        assert ref_bugs[0] in update.bugs
        assert ref_bugs[1] in update.bugs


    def test_add_criterion(self):
        db.session.add(self.ref_release)
        db.session.add(self.ref_milestone)
        db.session.commit()

        ref_criterion = "This is a fancy criterion that makes magic happen"
        ref_number = 3
        ref_current = True

        test_criterion = Criterion(ref_criterion, self.ref_milestone,
                                   ref_number)

        db.session.add(test_criterion)
        db.session.commit()

        criteria = Criterion.query.filter_by(number=ref_number).all()
        assert len(criteria) == 1
        criterion = criteria[0]

        assert criterion.criterion == ref_criterion
        assert criterion.number == ref_number
        assert criterion.current == ref_current
        assert criterion.milestone == self.ref_milestone
